@@ -1,8 +1,8 @@
 """
 ccp is a python library for calculation of centrifugal compressor performance.
 It is based on the book of {cite}`ludtke2004process` and uses
-[CoolProp](http://www.coolprop.org/)
-[REFPROP](https://www.nist.gov/srd/refprop)
+[CoolProp](http://www.coolprop.org/) :cite:`coolprop` and
+[REFPROP](https://www.nist.gov/srd/refprop) :cite:`refprop`
 for the gas properties calculations.
 
 ```{code-block} python
@@ -82,7 +82,7 @@ _library_path = _path / _shared_library
 if not _library_path.is_file():
     _warnings.warn(f"{_library_path}.\nREFPROP not configured.")
 
-__version__ = "0.1.10"
+__version__ = "0.1.13"
 
 __version__full = (
     f"ccp: {__version__} | "
@@ -93,16 +93,16 @@ __version__full = (
 ###############################################################################
 # pint
 ###############################################################################
-
-from pint import (
-    UnitRegistry as _UnitRegistry,
-    set_application_registry as _set_application_registry,
-)
+import pint as _pint
 
 _new_units = _Path(__file__).parent / "config/new_units.txt"
-ureg = _UnitRegistry()
-ureg.load_definitions(str(_new_units))
-_set_application_registry(ureg)
+ureg = _pint.get_application_registry()
+if isinstance(ureg.get(), _pint.registry.LazyRegistry):
+    ureg = _pint.UnitRegistry()
+    ureg.load_definitions(str(_new_units))
+    # set ureg to make pickle possible
+    _pint.set_application_registry(ureg)
+
 Q_ = ureg.Quantity
 _warnings.filterwarnings("ignore", message="The unit of the quantity is stripped")
 
